@@ -14,7 +14,7 @@ my $prefix = __PACKAGE__;
 # this module import config
 #
 use Carp ();
-use Net::SNMP::Mixin::Util qw/idx2val hex2octet normalize_mac push_error/;
+use Net::SNMP::Mixin::Util qw/idx2val hex2octet normalize_mac push_error get_init_slot/;
 
 #
 # this module export config
@@ -75,11 +75,11 @@ Net::SNMP::Mixin::Dot1dStp - mixin class for 802.1D spanning tree information
 
 =head1 VERSION
 
-Version 0.03
+Version 0.04
 
 =cut
 
-our $VERSION = '0.03';
+our $VERSION = '0.04';
 
 =head1 SYNOPSIS
 
@@ -91,6 +91,7 @@ our $VERSION = '0.03';
   $session->init_mixins;
 
   snmp_dispatcher();
+  $session->init_ok();
   die $session->errors if $session->errors;
 
   my $stp_group = $session->get_dot1d_stp_group;
@@ -167,8 +168,10 @@ The dot1dStpDesignatedRoot is a BridgeId struct of priority and MacAddress. The
 
 sub get_dot1d_stp_group {
   my $session = shift;
-  Carp::croak "'$prefix' not initialized,"
-    unless $session->{$prefix}{__initialized};
+  my $agent   = $session->hostname;
+
+  Carp::croak "$agent: '$prefix' not initialized,"
+    unless $session->init_ok($prefix);
 
   # just a shallow copy for shallow values
   my $result = { %{ $session->{$prefix}{dot1dStpGroup} } };
@@ -234,8 +237,10 @@ See also the L<< Net::SNMP::Mixin::Dot1dBase >> for a mixin to get the mapping b
 
 sub get_dot1d_stp_port_table {
   my $session = shift;
-  Carp::croak "'$prefix' not initialized,"
-    unless $session->{$prefix}{__initialized};
+  my $agent   = $session->hostname;
+
+  Carp::croak "$agent: '$prefix' not initialized,"
+    unless $session->init_ok($prefix);
 
   # stash for return values
   my $result = {};
@@ -321,11 +326,24 @@ Fetch the dot1dSTP related snmp values from the host. Don't call this method dir
 
 =cut
 
+#
+# due to the asynchron nature, we don't know what init job is really the last, we decrement
+# the value after each callback
+#
+use constant THIS_INIT_JOBS => 2;
+
 sub _init {
   my ( $session, $reload ) = @_;
 
-  die "$prefix already initalized and reload not forced.\n"
-    if $session->{$prefix}{__initialized} && not $reload;
+  my $agent = $session->hostname;
+
+  die "$agent: $prefix already initialized and reload not forced.\n"
+    if exists get_init_slot($session)->{$prefix}
+      && get_init_slot($session)->{$prefix} == 0
+      && not $reload;
+
+  # set number of async init jobs for proper initialization
+  get_init_slot($session)->{$prefix} = THIS_INIT_JOBS;
 
   # populate the object with needed mib values
   #
@@ -356,15 +374,11 @@ sub _fetch_dot1d_stp_group {
     -endindex => '14.0',
 
     # define callback if in nonblocking mode
-    $session->nonblocking
-    ? ( -callback => \&_dot1d_stp_group_cb )
-    : (),
+    $session->nonblocking ? ( -callback => \&_dot1d_stp_group_cb ) : (),
 
   );
 
   unless (defined $result) {
-    # Net::SNMP looses sometimes error messages in nonblocking
-    # mode, so we save them in an extra buffer
     my $err_msg = $session->error;
     push_error($session, "$prefix: $err_msg") if $err_msg;
     return;
@@ -390,10 +404,9 @@ sub _dot1d_stp_group_cb {
   my $vbl     = $session->var_bind_list;
 
   unless (defined $vbl) {
-    # Net::SNMP looses sometimes error messages in nonblocking
-    # mode, so we save them in an extra buffer
-    my $err_msg = $session->error;
-    push_error($session, "$prefix: $err_msg") if $err_msg;
+    if (my $err_msg = $session->error) {
+      push_error($session, "$prefix: $err_msg");
+    };
     return;
   }
 
@@ -437,7 +450,10 @@ sub _dot1d_stp_group_cb {
   $stash_ptr->{dot1dStpBridgeForwardDelay} =
     $vbl->{ DOT1D_STP_BRIDGE_FWD_DELAY() };
 
-  $session->{$prefix}{__initialized}++;
+  # this init job is finished
+  get_init_slot($session)->{$prefix}--;
+
+  return 1;
 }
 
 =head2 B<< _fetch_dot1d_stp_port_tbl($session) >>
@@ -462,8 +478,7 @@ sub _fetch_dot1d_stp_port_tbl {
 
     # define callback if in nonblocking mode
     $session->nonblocking
-    ? ( -callback => \&_dot1d_stp_port_tbl_cb )
-    : (),
+    ? ( -callback => \&_dot1d_stp_port_tbl_cb ) : (),
   );
 
   unless (defined $result) {
@@ -494,10 +509,9 @@ sub _dot1d_stp_port_tbl_cb {
   my $vbl     = $session->var_bind_list;
 
   unless (defined $vbl) {
-    # Net::SNMP looses sometimes error messages in nonblocking
-    # mode, so we save them in an extra buffer
-    my $err_msg = $session->error;
-    push_error($session, "$prefix: $err_msg") if $err_msg;
+    if (my $err_msg = $session->error) {
+      push_error($session, "$prefix: $err_msg");
+    };
     return;
   }
 
@@ -527,7 +541,10 @@ sub _dot1d_stp_port_tbl_cb {
   $stash_ptr->{dot1dStpPortForwardTransitions} =
     idx2val( $vbl, DOT1D_STP_PORT_FORWARD_TRANSITIONS );
 
-  $session->{$prefix}{__initialized}++;
+  # this init job is finished
+  get_init_slot($session)->{$prefix}--;
+
+  return 1;
 }
 
 =head2 B<< _unpack_bridge_id($bridgeId) >>
@@ -608,7 +625,7 @@ Karl Gaissmaier <karl.gaissmaier at uni-ulm.de>
 
 =head1 COPYRIGHT & LICENSE
 
-Copyright 2008 Karl Gaissmaier, all rights reserved.
+Copyright 2008-2016 Karl Gaissmaier, all rights reserved.
 
 This program is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
